---
title: Triangulation de Delaunay
date: 2020-02-19
---

## Généralités

* Soit un ensemble de $N$ points $\mathcal{P}=\{p_i\}_{i=1}^N$.
* Une triangulation est un ensemble de triangle dont les sommets sont formés des points de $\mathcal{P}$ et couvrant leur envelope convexe.

![Deux exemples de triangulation (source: [wikipedia](https://bit.ly/2P2XCJ9)).](figs/PointSetTriangulations.svg){width=100%}

## La triangulation de Delaunay

* La triangulation de Delaunay (TD) d'un ensemble de points $\mathcal{P}$ est telle qu'aucun point de $\mathcal{P}$ ne se trouve dans le cercle circonscrit d'un triangle de la TD.
* Les points sont tous sur la surface de chaque cercle circonscrit des triangles de la TD.
* Elle maximise le plus petit angle de tous les triangles (on essaie d'éviter les triangle avec les angles très petits).
* Il existe plusieurs algorithmes pour construire cette triangulation: *flip algorithm*, *incremental edge flip*, *Bowyer--Watson*, ...

## L'algorithme de Bowyer--Watson

* Dans cet algorithme on commence par construire un **super-triangle** contenant tous les points de $\mathcal{P}$.
* On ajoute ensuite les points itérativement:
  * Chaque fois qu'on ajoute un point, $\vec p$, on retire tous les triangles dont le triangle circonscrit contient le point $\vec p$.
  * On se retrouve avec avec un "trou" qui contient $\vec p$.
  * Toutes les arrêtes du polygones sont reliées à $\vec p$ pour former les nouveau triangles à ajouter à la triangulation.
* On retire tous les triangles qui contiennent les points du super-triangle.


## Exemple d'ajouts (1/6) (source [wikipedia](https://bit.ly/2vJs5p2))

![Ajout du premier point dans le super-triangle](figs/Bowyer-Watson_0.png){width=100%}

## Exemple d'ajouts (2/6) (source [wikipedia](https://bit.ly/2vJs5p2))

![Ajout du premier point dans le super-triangle](figs/Bowyer-Watson_1.png){width=100%}

## Exemple d'ajouts (3/6) (source [wikipedia](https://bit.ly/2vJs5p2))

![Ajout du premier point dans le super-triangle](figs/Bowyer-Watson_2.png){width=100%}

## Exemple d'ajouts (4/6) (source [wikipedia](https://bit.ly/2vJs5p2))

![Ajout du premier point dans le super-triangle](figs/Bowyer-Watson_3.png){width=100%}

## Exemple d'ajouts (5/6) (source [wikipedia](https://bit.ly/2vJs5p2))

![Ajout du premier point dans le super-triangle](figs/Bowyer-Watson_4.png){width=100%}

## Exemple d'ajouts (6/6) (source [wikipedia](https://bit.ly/2vJs5p2))

![Ajout du premier point dans le super-triangle](figs/Bowyer-Watson_6.png){width=100%}

<!-- ## Pseudo-code de l'algorithme (1/2)

\scriptsize

```C
void bowyer_watson(point *points, int num_p, vec_triangle *t, int *num_t) {
    ajout_super_triangle(vec_triangle, num_t); // ajout super triangle
    
    pour chaque p dans points {
        mauvais_triangles = vecteur_vide();
        pour chaque t dans vec_triangle {
            si p est dans le triangle circonscrit de t {
                vecteur_push(mauvais_triangles, t);
            }
        }

        vec_aretes = vecteur_vide();
        pour chaque t dans mauvais_triangles {
            pour chaque arete dans t {
                si l'arete n'est partagée avec aucun triangle de mauvais_triangles {
                    vecteur_push(vec_aretes, arete);
                }
            }
        }

        pour chaque t dans mauvais_triangles {
            vec_remove(vec_triangles, t);
        }

        pour chaque arete dans vec_aretes {
            t = form_triangle_from_edge_and_point(arete, p);
            vector_push(vec_triangles, t);
        }

        pour chaque t dans vec_triangles {
            if t contient un point du super triangle {
                vec_remove(vec_triangles, t);
            }
        }
    }
}
``` -->

## Pseudo-code de l'algorithme (1/2)

\scriptsize

```C
void bowyer_watson(vec_points *points, vec_triangle *triangles) {
    ajout_super_triangle(points, t, num_t); // ajout super triangle
    
    pour chaque p dans points {
        mauvais_triangles = vecteur_vide();
        // on détermine les mauvais triangles
        pour chaque t dans triangles {
            si p est dans le triangle circonscrit de t {
                vecteur_push(mauvais_triangles, t);
            }
        }
        vec_aretes = vecteur_vide();
        // stockage du polygone entourant le "trou"
        pour chaque t dans mauvais_triangles {
            pour chaque arete dans t {
                si l'arete n'est partagée avec aucun triangle de mauvais_triangles {
                    vecteur_push(vec_aretes, arete);
                }
            }
        }
        // on retire les mauvais triangles
        pour chaque t dans mauvais_triangles {
            vec_remove(triangles, t);
        }
```

## Pseudo-code de l'algorithme (2/2)

\scriptsize

```C
        // on forme les nouveaux triangles et on les ajoute
        pour chaque arete dans vec_aretes {
            t = form_triangle_from_edge_and_point(arete, p);
            vector_push(triangles, t);
        }
    }

    // on retire les points du super triangle
    pour chaque t dans triangles {
        if t contient un point du super triangle {
            vec_remove(triangles, t);
        }
    }
}
```


## Appartenance au cercle circonscrit

* Une partie fondamentale de l'algorithme réside dans la capacité à déterminer si un point est dans le cercle circonscrit.
* On peut déterminer si un point $D$ est dans le triangle circonscrit délimité par les points $A$, $B$, et $C$ (qui sont ordonnés dans le sens inverse des aiguilles d'une montre), si
$$
\begin{vmatrix}A_{x}-D_{x}&A_{y}-D_{y}&(A_{x}-D_{x})^{2}+(A_{y}-D_{y})^{2}\\B_{x}-D_{x}&B_{y}-D_{y}&(B_{x}-D_{x})^{2}+(B_{y}-D_{y})^{2}\\C_{x}-D_{x}&C_{y}-D_{y}&(C_{x}-D_{x})^{2}+(C_{y}-D_{y})^{2}\end{vmatrix}>0,
$$
où $|.|$ signifie le calcul du déterminant.
* Une façon alternative est donnée dans l'énoncé du travail pratique. Choisissez celle que vous préférez.

